@@ -396,12 +396,7 @@ namespace libsemigroups {
     class PrintTable {
      public:
       explicit PrintTable(size_t width = 72)
-          : _rows(), _header(), _footer(), _width(width) {
-        try {
-          std::locale::global(std::locale("en_US.UTF-8"));
-        } catch (std::runtime_error const& e) {
-        }
-      }
+          : _rows(), _header(), _footer(), _width(width) {}
 
       std::string emit() {
         std::string result = lineohash() + _header + lineohash();
@@ -467,6 +462,10 @@ namespace libsemigroups {
     };
   }  // namespace detail
 
+  namespace report {
+    bool should_report() noexcept;
+  }
+
   //! This struct can be used to enable printing of some information during
   //! various of the computation in ``libsemigroups``. Reporting is enable (or
   //! not) at construction time, and disable when the ReportGuard goes out of
